/* ARCADEmini simulator DLL
 * version 1.2 date 2003-10-31
 * part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
 * Copyright (C) 2003 sphaera <sphaera@geekness.de>
 *                and 1stein <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <windows.h>

#include "am_dev.h"

//-------------------------------------------------------------------------------------------
// get information about simulator
//-------------------------------------------------------------------------------------------
void _stdcall ArcadeMiniSimInfo( char * pBuffer, unsigned long BufferLen )
{
	if( BufferLen > 0 )
	{
		memset( pBuffer, 0, BufferLen );
		strncpy( pBuffer, "ARCADEmini simulator DLL\n"
			              "version 1.2 date 2003-10-31\n"
						  "part of the ARCADEmini project - http[s]://arcademini.schuermans.info/\n"
						  "Copyright (C) 2003 sphaera <sphaera@geekness.de>\n"
						  "                and 1stein <1stein@schuermans.info>\n"
						  "Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html\n",
						  BufferLen - 1 );
	}
}

//-------------------------------------------------------------------------------------------
// initialize a simulator
//-------------------------------------------------------------------------------------------
void * _stdcall ArcadeMiniSimInit( char * Filename )
{
	FILE * p_file;
	st_am_state * p_state;

	//open the file
	if( Filename == NULL || Filename[0] == 0 )
		return NULL;
	p_file = fopen( Filename, "rb" );
	if( p_file == NULL )
		return NULL;

	//allocate the structure for the state
	p_state = (st_am_state *)malloc( sizeof( st_am_state ) );
	if( p_state == NULL )
	{
		fclose( p_file );
		return NULL;
	}

	//initialize state
	if( am_dev_init_state( p_state, p_file, NULL ) != 0 )
	{
		free( p_state );
		fclose( p_file );
		return NULL;
	}

	//return pointer to state
	return (void *)p_state;
}

//-------------------------------------------------------------------------------------------
// initialize a simulator with log file
//-------------------------------------------------------------------------------------------
void * _stdcall ArcadeMiniSimInitLog( char * Filename, char * LogFilename )
{
	FILE * p_file, * p_log_file;
	st_am_state * p_state;

	//open the file
	if( Filename == NULL || Filename[0] == 0 )
		return NULL;
	p_file = fopen( Filename, "rb" );
	if( p_file == NULL )
		return NULL;

	//open the logfile
	if( LogFilename == NULL || LogFilename[0] == 0 )
	{
		fclose( p_file );
		return NULL;
	}
	p_log_file = fopen( LogFilename, "wt" );
	if( p_log_file == NULL )
	{
		fclose( p_file );
		return NULL;
	}

	//allocate the structure for the state
	p_state = (st_am_state *)malloc( sizeof( st_am_state ) );
	if( p_state == NULL )
	{
		fclose( p_log_file );
		fclose( p_file );
		return NULL;
	}

	//initialize state
	if( am_dev_init_state( p_state, p_file, p_log_file ) != 0 )
	{
		free( p_state );
		fclose( p_log_file );
		fclose( p_file );
		return NULL;
	}

	//return pointer to state
	return (void *)p_state;
}

//-------------------------------------------------------------------------------------------
// cleanup a simulator
//-------------------------------------------------------------------------------------------
void _stdcall ArcadeMiniSimExit( void * p_sim )
{
	st_am_state * p_state;

	//get pointer to state
	p_state = (st_am_state *)p_sim;
	if( p_state == NULL )
		return;

	//close logfile
	if( p_state->p_log_file != NULL )
		fclose( p_state->p_log_file );

	//close file
	fclose( p_state->p_file );
}

//-------------------------------------------------------------------------------------------
// do a step
// (returns number of milliseconds of current delay or 0 if not waiting for a delay)
//-------------------------------------------------------------------------------------------
unsigned short _stdcall ArcadeMiniSimStep( void * p_sim )
{
	st_am_state * p_state;

	//get pointer to state
	p_state = (st_am_state *)p_sim;
	if( p_state == NULL )
		return 0;

	//execute an instruction and return rest of delay
	return am_dev_exec( p_state );
}

//-------------------------------------------------------------------------------------------
// get the current picture
// picture must point to a buffer with 520 bytes
// returns 0 if no new picture is available
//-------------------------------------------------------------------------------------------
int _stdcall ArcadeMiniSimGetPicture( void * p_sim, char * p_picture )
{
	st_am_state * p_state;
	int i, x, y;

	//get pointer to state
	p_state = (st_am_state *)p_sim;
	if( p_state == NULL )
		return 0;

	//no new picture
	if( !p_state->picture_changed )
		return 0;
	p_state->picture_changed = 0;

	//copy current picture into buffer
	for( y = 0, i = 0; y < 20; y++ )
		for( x = 0; x < 26; x++, i++ )
			p_picture[i] = p_state->picture[y][x];

	//return success
	return 1;
}

//-------------------------------------------------------------------------------------------
// set the current state of the keys
//-------------------------------------------------------------------------------------------
void _stdcall ArcadeMiniSimSetKeyState( void * p_sim, unsigned char keystate )
{
	st_am_state * p_state;

	//get pointer to state
	p_state = (st_am_state *)p_sim;
	if( p_state == NULL )
		return;

	//set new keystate
	am_dev_keystate( p_state, keystate );
}

//-------------------------------------------------------------------------------------------
// set the current state of the snes keys
//-------------------------------------------------------------------------------------------
void _stdcall ArcadeMiniSimSetSnesKeyState( void * p_sim, unsigned char snes_no, unsigned short keystate )
{
	st_am_state * p_state;

	//get pointer to state
	p_state = (st_am_state *)p_sim;
	if( p_state == NULL )
		return;

	//set new snes keystate
	am_dev_snes_keystate( p_state, snes_no, keystate );
}
